/* ========================================================================= *
 * Array generator
 * ========================================================================= */

#ifndef _ARRAY_H_
#define _ARRAY_H_

#include <stddef.h>


/* ------------------------------------------------------------------------- *
 * Create a nearly sorted array of integers (from 0 to length-1).
 *
 * The array must later be deleted by calling free().
 *
 * PARAMETERS
 * length       Number of elements in the array (pre-condition: 0 < length)
 * nbSwaps      Number of uniform swaps
 *              If 0, the array is sorted
 *              If low, the array is nearly sorted
 *
 * RETURN
 * array        A new array of integers, or NULL in case of error
 * ------------------------------------------------------------------------- */
int* createNearlySortedArray(size_t length, unsigned int nbSwaps);

/* ------------------------------------------------------------------------- *
 * Create a random array of integers (from 0 to length-1).
 *
 * The array must later be deleted by calling free().
 *
 * PARAMETERS
 * length       Number of elements in the array (pre-condition: 0 < length)
 *
 * RETURN
 * array        A new array of integers, or NULL in case of error
 * ------------------------------------------------------------------------- */
int* createRandomArray(size_t length);

/* ------------------------------------------------------------------------- *
 * Create a random array of a given size with few uniques integers
 *
 * The array must later be deleted by calling free().
 *
 * PARAMETERS
 * length       Number of elements in the array (pre-condition: 0 < length)
 * nbUniques    Number of unique elements (truncated at length)
 *
 * RETURN
 * array        A new array of integers, or NULL in case of error
 * ------------------------------------------------------------------------- */
int* createFewUniquesArray(size_t length, unsigned int nbUniques);


#endif // !_ARRAY_H_
