/* ========================================================================= *
 * FrozenList interface
 * ========================================================================= */

#ifndef _FROZEN_LIST_H_
#define _FROZEN_LIST_H_

 #include <stddef.h>

/* Opaque Structure */
typedef struct node_t FrozenList;

/* ------------------------------------------------------------------------- *
 * Create a FrozenList containing the elements of the given array
 *
 * The FrozenList must later be deleted by calling freeFrozenList().
 *
 * PARAMETERS
 * array        The integers to build the list
 * length       Number of elements in the array (pre-condition: 0 < length)
 *
 * RETURN
 * flist        A pointer to the FrozenList, or NULL in case of error
 * ------------------------------------------------------------------------- */
FrozenList* createFrozenList(const int* array, size_t length);


/* ------------------------------------------------------------------------- *
 * Free the allocated memory of the FrozenList
 *
 * PARAMETERS
 * flist       A valid pointer to a FrozenList object
 * ------------------------------------------------------------------------- */
void freeFrozenList(FrozenList* flist);

/* ------------------------------------------------------------------------- *
 * Return (a copy of) the data this FrozenList head contains
 *
 * PARAMETERS
 * flist       A valid pointer to a FrozenList object
 *
 * RETURN
 * data        The integer contained by this FrozenList head
 * ------------------------------------------------------------------------- */
int getData(const FrozenList* flist);

/* ------------------------------------------------------------------------- *
 * Return a pointer to the tail of the FrozenList (the FrozenList starting
 * at the next node)
 *
 * PARAMETERS
 * flist       A valid pointer to a FrozenList object
 *
 * RETURN
 * next        A pointer to the tail of the FrozenList or NULL if the tail
 *             is empty
 * ------------------------------------------------------------------------- */
FrozenList* getTail(FrozenList* flist);


/* ------------------------------------------------------------------------- *
 * Change the tail of this FrozenList
 *
 * PARAMETERS
 * flist       A valid pointer to a FrozenList object
 * newTail     A valid pointer to a FrozenList object which will become the new
 *             tail
 *
 * RETURN
 * prevTail    A pointer to the previous tail of the FrozenList or NULL if
 *             the tail was previously empty
 * ------------------------------------------------------------------------- */
FrozenList* setTail(FrozenList* flist, FrozenList* newTail);


#endif // !_FROZEN_LIST_H_
