#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <math.h>

#include "Array.h"
#include "SortArray.h"

static const size_t DISPLAY_LIMIT = 10;


/* ------------------------------------------------------------------------- *
 * Print the given array on stdout.
 *
 * PARAMETERS
 * array        Array to sort
 * length       Number of elements in the array
 * ------------------------------------------------------------------------- */
static void printArray(const int* array, size_t length)
{
    printf("[");
    for(size_t i=0; i<length; i++)
        printf("%d, ", array[i]);
    printf("]\n");
}


/* ------------------------------------------------------------------------- *
 * Compute the CPU time (in seconds) used by the sortArray function.
 *
 * PARAMETERS
 * array        Array to sort
 * length       Number of elements in the array
 * ------------------------------------------------------------------------- */
static void timeSorting(int* array, size_t length)
{
    //Print small array
    if(length <= DISPLAY_LIMIT)
    {
        printf("Array to be sorted:\n");
        printArray(array, length);
    }

    //Time the actual sorting
    clock_t start = clock();
    sortArray(array, length);
    double sec = ((double) (clock() - start)) / CLOCKS_PER_SEC;

    //Print small array
    if(length <= DISPLAY_LIMIT)
    {
        printf("Sorted array:\n");
        printArray(array, length);
    }

    //Print sorting time
    printf("Sorting time: %f\n", sec);
}





/* ------------------------------------------------------------------------- *
 * Main
 * ------------------------------------------------------------------------- */
int main(int argc, char** argv)
{


    /* ---------- Command line part -----------*/
    // Parsing command line arguments
    if (argc != 2)
    {
        fprintf(stderr, "Usage: %s <array size>\n", argv[0]);
        exit(EXIT_FAILURE);
    }
    size_t length = (size_t) atoi(argv[1]);

    /* ---------- Main program -----------*/
    srand(time(NULL));//Use an integer seed to get a fix sequence. Ex : srand(4)

    printf("Size of the array: %d\n", (int) length);

    int* randomArray = createRandomArray(length);
    printf("--Random array:\n");
    timeSorting(randomArray, length);
    free(randomArray);

    unsigned int nbSwaps = (unsigned int)sqrt((double)length);
    int* sortedArray = createNearlySortedArray(length, nbSwaps);
    printf("--Nearly sorted array:\n");
    timeSorting(sortedArray, length);
    free(sortedArray);

    unsigned int nbUniques = (unsigned int)sqrt((double)length);
    int* fewUniquesArray = createFewUniquesArray(length, nbUniques);
    printf("--Few uniques array:\n");
    timeSorting(fewUniquesArray, length);
    free(fewUniquesArray);

    return 0;
}
