#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <math.h>

#include "Array.h"
#include "FrozenList.h"
#include "SortFrozenList.h"

static const size_t DISPLAY_LIMIT = 10;

/* ------------------------------------------------------------------------- *
 * Print the given FrozenList on stdout.
 *
 * PARAMETERS
 * flist        A valid pointer to a FrozenList object
 * ------------------------------------------------------------------------- */
static void printFrozenList(FrozenList* flist)
{
    while(flist != NULL)
    {
        printf("[%d]->", getData(flist));
        flist = getTail(flist);
    }
    printf("X\n");
}


/* ------------------------------------------------------------------------- *
 * Compute the CPU time (in seconds) used by the sortFrozenList function.
 *
 * PARAMETERS
 * flist        A valid pointer to a FrozenList object
 * length       The length of the FrozenList
 *
 * RETURN
 * flist        A valid pointer to the first element of the FrozenList
 * ------------------------------------------------------------------------- */
static FrozenList* timeSorting(FrozenList* flist, size_t length)
{
    //Print small flist
    if(length <= DISPLAY_LIMIT)
    {
        printf("FrozenList to sort:\n");
        printFrozenList(flist);
    }

    //Time the actual sorting
    clock_t start = clock();
    flist = sortFrozenList(flist);
    double sec = ((double) (clock() - start)) / CLOCKS_PER_SEC;

    //Print small array
    if(length <= DISPLAY_LIMIT)
    {
        printf("Sorted FrozenList:\n");
        printFrozenList(flist);
    }

    //Print sorting time
    printf("Sorting time: %f\n", sec);
    return flist;
}





/* ------------------------------------------------------------------------- *
 * Main
 * ------------------------------------------------------------------------- */
int main(int argc, char** argv)
{


    /* ---------- Command line part -----------*/
    // Parsing command line arguments
    if (argc != 2)
    {
        fprintf(stderr, "Usage: %s <FrozenList size>\n", argv[0]);
        exit(EXIT_FAILURE);
    }
    size_t length = (size_t) atoi(argv[1]);

    /* ---------- Main program -----------*/
    srand(time(NULL));//Use an integer seed to get a fix sequence

    printf("Size of the FrozenList: %d\n", (int) length);

    int* randomArray = createRandomArray(length);
    FrozenList* randomList = createFrozenList(randomArray, length);
    free(randomArray);
    printf("--Random FrozenList:\n");
    randomList = timeSorting(randomList, length);
    freeFrozenList(randomList);

    unsigned int nbSwaps = (unsigned int)sqrt((double)length);
    int* sortedArray = createNearlySortedArray(length, nbSwaps);
    FrozenList* sortedList = createFrozenList(sortedArray, length);
    free(sortedArray);
    printf("--Nearly sorted FrozenList:\n");
    sortedList = timeSorting(sortedList, length);
    freeFrozenList(sortedList);

    unsigned int nbUniques = (unsigned int)sqrt((double)length);
    int* fewUniquesArray = createFewUniquesArray(length, nbUniques);
    FrozenList* fewUniquesList = createFrozenList(fewUniquesArray, length);
    free(fewUniquesArray);
    printf("--Few uniques FrozenList:\n");
    fewUniquesList = timeSorting(fewUniquesList, length);
    freeFrozenList(fewUniquesList);

    return 0;
}
